from vpython import *
#GlowScript 3.0 VPython

scene.width = 1900
scene.height = 1000
scene.forward = vector(-0.8, -0.6, -1)

G = 6.67e-11 # SI units
Me = 6e24 # mass of Earth kg

R = 6371e3 #radius of Earth metres

theta = 55.86/180.0*pi # laltitude of Glasgow
phi = -4.25/180.0*pi   # longitude of Glasgow

dt0 = 5 # standard iteration jump interval in seconds
dt = dt0 # current jump interval
hard_surface = True
spin = False

v0 = 7e3 # default launch speed m/s
alpha = 35.0 # default firing angle from vertical in degrees

Earth = sphere(radius=R,texture="https://mw1.google.com/mw-earth-vectordb/kml-samples/bmng12/files/BMNG-Mar.jpg")
#Earth = sphere(radius=R,texture="https://i.imgur.com/b9OPaHC.jpg")
projectile = sphere(p=0, radius=R/50, color=color.yellow, make_trail=True)
resetscene()


###################################################
def resetscene(): # reset the scene
   global projectile,  dt
   projectile.clear_trail() 
   z = cos(theta)*cos(phi)
   x = cos(theta)*sin(phi)
   y = sin(theta)
   projectile.pos=1.001*R*vector(x,y,z)
   projectile.p = v0*rotate(vector(x,y,z), angle=alpha/180.0*pi, axis=vector(1,0,0)) # unit mass object
   dt = dt0
   escape_velocity = sqrt(G*Me*2/R)
   scene.title = "University of Glasgow A1 Dynamical Astronomy.     Projectile fired at " + round(1000.0*v0/escape_velocity)/1000.0 + " escape speed, " + round(alpha) + " degrees from the vertical.           "  + "r: reset;     t: toggle transparency;     arrow up/down: faster/slower;     arrow left/right: tilt south/north;     s: toggle Earth spin"


#####################################################
def keyInput(evt):
    global alpha, projectile, R, v0, theta, phi, hard_surface, spin
    s = evt.key
    if (s == 't'): #toggle earth transparency
        if Earth.opacity == 0.8:
            Earth.opacity = 1.0
            hard_surface = True
        else:
            Earth.opacity = 0.8
            hard_surface = False
            dt = dt0

    if (s == 'r'): # reset the scene  
        resetscene()
        
    if (s == 'up'): ##faster 
        v0 = v0*1.1
        resetscene()

    if (s == 'down'): ##faster 
        v0 = v0/1.1
        resetscene()

    if (s == 'left'): ##faster 
        alpha = alpha+5
        resetscene()

    if (s == 'right'): ##faster 
        alpha = alpha - 5
        resetscene()
        
    if (s == 's'): ##faster 
        spin = not spin

scene.bind('keydown', keyInput)


#####################################################
def dydt(r,v): #derivatives
    deriv = [vector(0.0,0.0,0.0)] * 2
    deriv[1] = -G*Me*norm(r)/mag(r)**2* dt
    deriv[0] = v * dt
    return deriv  

#####################################################
  
while True: # solve using 4th order Runge Kutta
    rate(200)
    k1 = dydt(projectile.pos, projectile.p)
    k2 = dydt(projectile.pos + 0.5*k1[0], projectile.p + 0.5*k1[1])
    k3 = dydt(projectile.pos + 0.5*k2[0], projectile.p + 0.5*k2[1])
    k4 = dydt(projectile.pos +     k3[0], projectile.p +     k3[1])
    projectile.pos += 1.0/6.0*(k1[0] + k4[0]) + 1.0/3.0*(k2[0] + k3[0])
    projectile.p += 1.0/6.0*(k1[1] + k4[1]) + 1.0/3.0*(k2[1] + k3[1])
 
    if hard_surface and (mag(projectile.pos) < R):
        dt = 0
    if spin:
        Earth.rotate(angle = dt/(24.0*3600)*2*pi, axis = vector(0,1,0))
 
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  
  